import emoji
import csv


emoji_sent = dict()
with open('/Volumes/cchen224/emoji.csv', 'r') as i:
    csvreader = csv.reader(i)
    for row in csvreader:
        strs = ':' + '_'.join(row[2].split()).lower() + ':'
        uni = emoji.emojize(strs)
        if not uni.endswith(':'):
            emoji_sent[uni] = float(row[1])

with open('/Users/cchen224/Downloads/data/face', 'r') as i:
    face = dict(map(lambda (w,m): (w, float(m) / 4.), [wmsr.strip().split('\t')[0:2] for wmsr in i ]))

emoji_sent.update(face)


import csv
import pickle
import re
from glob import glob

import matlab.engine
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.feature_extraction.text import ENGLISH_STOP_WORDS
from sklearn.feature_extraction.text import TfidfTransformer
from sklearn.decomposition import NMF
from sklearn.pipeline import Pipeline
import os.path
from datetime import datetime

from CMUTweetTagger import runtagger_parse

STOPWORDS = ENGLISH_STOP_WORDS.union(['hi', 'just'])

# eng = matlab.engine.start_matlab()




# PATTERN = re.compile('(\\@|http(s|)\\:\\/\\/|pic\\.twitter\\.com)\S+')
INCLUDE = dict(map(lambda x: (x, ''), ['V', 'N', 'A', 'R', '!', 'U', 'G', 'D', '^', 'T', '#', 'S', 'Z', 'L', 'E', '$']))
file_counter = 0
for file in sorted(glob('/Volumes/cchen224/brand_threads/input_0615/*.csv.txt'), key=os.path.getsize)[::-1]:
    file_counter += 1
    print '\n\n\n' + str(file_counter) + '.',
    brand_name = file.split('/')[-1][:-8]
    print brand_name, datetime.now().strftime('%H:%M:%S')

    for type in ['brand', 'user']:
        print '\ngetting data...',
        t0 = datetime.now()
        tweets = []
        tttttt = []
        with open(file, 'r') as i:
            csvreader = csv.reader(i, delimiter='\t')
            posters = dict()
            uids = []
            tids = []
            for row in csvreader:
                if row:
                    poster = row[2]
                    uids.append(poster)
                    tids.append(row[1])
                    text = re.sub('/n|/r', ' ', row[3].strip())
                    if type == 'brand':
                        if poster not in posters:
                            posters[poster] = text
                        else:
                            posters[poster] += '. ' + text
                    if type == 'user':
                        if poster not in posters:
                            posters[poster] = text

                else:
                    # if (len(posters) == 2 and brand_name in posters) or (len(posters) == 1):
                    #     for poster in posters:
                    #         # tweets.append(PATTERN.sub(' ', posters[poster]))
                    #         tweets.append(posters[poster])


                    # brand only agreigte
                    if uids[0] != brand_name:
                        if type == 'brand':
                            if (brand_name in posters and len(posters) == 2):
                                for poster in posters:
                                    if brand_name == poster:
                                        if posters[poster]:
                                            tweets.append(posters[poster])
                                            tttttt.append(tids[0])

                        # user only
                        if type == 'user':
                            if (brand_name in posters and len(posters) == 2):# or (brand_name not in posters and len(posters) == 1):
                                # tweets.append(PATTERN.sub(' ', posters[poster]))
                                for poster in posters:
                                    if poster != brand_name:
                                        if posters[poster]:
                                            tweets.append(posters[poster])
                                            tttttt.append(tids[0])
                    posters = dict()
                    uids = []
                    tids = []
        print 'done'

        print 'postagging...',
        postags = runtagger_parse(tweets)
        tweets_p = [' '.join([token[0].split('/')[-1] for token in tweet if token[1] in INCLUDE]) for tweet in postags]
        tweets_p = [re.sub('([0-9])[\\(|\\)|\\-|\\+]([0-9])', '\\1\\2', item) for item in tweets_p]
        print 'done'

        # for file in glob('/Volumes/cchen224/brand_threads/expended_thread_aws/@2KSupport.csv.txt'):
        #     with open(file, 'r') as i:
        #         doc_index.append(file.split('/')[-1])
        #         tweets.append(re.sub('(\\@|http(s|)\\:\\/\\/|pic\\.twitter\\.com)\S+', '', i.next()).strip())

        model = Pipeline([
            ('wc', CountVectorizer(stop_words=STOPWORDS)),
            ('tfidf', TfidfTransformer(use_idf=True))
        ])

        print 'tfidf convertion...',
        tfidf = model.fit_transform(tweets_p)
        # tfidf_d = tfidf.todense().tolist()
        # tfidf_m = matlab.double(tfidf_d)
        print tfidf.shape, 'done'

        # print 'nmf...',
        # W, H = eng.nmfsc(tfidf_m, 10., 0.5, 0.2, 'out', False, nargout=2)
        # print 'done'
        #
        #
        # W = [list(line) for line in W]
        # H = [list(line) for line in H]

        vocab = model.steps[0][1].vocabulary_
        vocab_= dict((v, k) for k, v in vocab.iteritems())

        print 'saving matrices...',
        # with open(file[:-8] + '_' + type + '_W.data', 'w') as o:
        #     pickle.dump(W, o)
        # with open(file[:-8] + '_' + type + '_H.data', 'w') as o:
        #     pickle.dump(H, o)
        with open(file[:-8] + '_' + type + '_V.data', 'w') as o:
            pickle.dump(tfidf, o)
        with open(file[:-8] + '_' + type + '_vocab.data', 'w') as o:
            pickle.dump(vocab_, o)
        with open(file[:-8] + '_' + type + '_docs.data', 'w') as o:
            pickle.dump(tweets, o)
        with open(file[:-8] + '_' + type + '_stats.data', 'w') as o:
            o.write(str(tfidf.shape))
            # pickle.dump(tfidf.shape, o)
        with open(file[:-8] + '_' + type + '_tids.data', 'w') as o:
            pickle.dump(tttttt, o)
        print 'done'
        print (datetime.now() - t0).seconds


        # import numpy as np
        # print type, ':'
        # for counter, topic in enumerate(H):
        #     print 'Topic' + str(counter) + ':',
        #     for index in np.array(topic).argsort()[-10:][::-1]:
        #         print vocab_[index],
        #     print
        #
        # for counter, topic in enumerate(W):
        #     # print 'Topic' + str(counter) + ':',
        #     print tweets[counter]
        #     for index in np.array(topic).argsort()[-5:][::-1]:
        #         print '\t', index, np.array(topic)[index]
        #     print



# eng.quit()


################################################################################################
import pickle
import numpy as np
H_cutoff = 0.2
with open('/Volumes/cchen224/brand_threads/input_0615/2ksupport_user_H.data', 'r') as i:
    H = np.array(pickle.load(i))
topics = [[index for index in np.array(topic).argsort()[-10:][::-1]] for topic in H]
topics_g = dict(enumerate([0]*10))
for j, topic in enumerate(topics):
    for k in xrange(j + 1, 10):
        if is_general(jaccard(topics[j], topics[k]), H_cutoff):
            topics_g[j] += 1
            topics_g[k] += 1
        else:
            topics_g[j] -= 1
            topics_g[k] -= 1

with open('/Volumes/cchen224/brand_threads/input_0615/2ksupport_user_W.data', 'r') as i:
    W = np.array(pickle.load(i))
    general = [sum(topics_g[c] * p for c, p in enumerate(t) if topics_g[c] > 0) for t in W]
    specifc = [sum(topics_g[c] * p for c, p in enumerate(t) if topics_g[c] < 0) for t in W]



def jaccard(x, y):
    x = set(x)
    y = set(y)
    return float(len(x & y)) / len(x | y)

def is_general(j_score, threshold):
    return True if j_score >= threshold else False


################################################################################################
cutoff = 0.1
ntopic = 10
counter = 0
for file in glob('/Volumes/cchen224/brand_threads/input_0615/*_brand_V.data'):
    counter += 1
    print '\n\n\n', str(counter) + '.', brand_fp

    brand_fp = re.sub('_V[.]data', '', file)

    print 'loading...',
    with open(file, 'r') as i:
        V = pickle.load(i)

    print 'modeling...',

    if V.shape[0] < ntopic or V.shape[1] < ntopic: continue
    nmf = NMF(init="nndsvd", n_components=ntopic, random_state=1, alpha=.1, l1_ratio=.8).fit(V)
    W = nmf.fit_transform(V)
    H = nmf.components_
    # len([item for item in H[0] if item != 0])

    top10 = [topic.argsort()[-ntopic:][::-1] for topic in H]

    # with open(brand_fp + '_vocab.data', 'r') as i:
    #     vocab_ = pickle.load(i)
    # top10w = [['_'.join(vocab_[index].split(' ')) for index in topic.argsort()[-10:][::-1]] for topic in H]
    # for line in top10w:
    #     for word in line:
    #         print word,
    #     print

    jd = dict()
    for j, topic in enumerate(top10):
        for k in xrange(j + 1, ntopic):
            jd[(j, k)] = jaccard(topic, top10[k])
    js = set()
    for pair in jd:
        if jd[pair] > cutoff:
            js.add(pair[0])
            js.add(pair[1])
    jc = set(range(ntopic)) - js

    print 'saving...',
    is_general = [sum([line[item] for item in js]) >= sum([line[item] for item in jc]) for line in W]
    with open(brand_fp + '_result.data', 'w') as o:
        pickle.dump(is_general, o)
    print 'done'
    # specifc = sum([W[0][item] for item in jc])
print 'ddd!'
################################################################################################

is_general = dict()
is_general_tids = dict()
for file in glob('/Volumes/cchen224/brand_threads/input_0615/*result.data'):
    with open(file, 'r') as i:
        is_general[file.split('/')[-1][:-12]] = pickle.load(i)
for file in glob('/Volumes/cchen224/brand_threads/input_0615/*tids.data'):
    with open(file, 'r') as i:
        is_general_tids[file.split('/')[-1][:-10]] = pickle.load(i)

import csv
c = 0
with open('/Volumes/cchen224/brand_threads/a.csv', 'w') as o:
    csvwriter = csv.writer(o)
    csvwriter.writerow(['brand', 'tid', 'user', 'brand_reply'])
    for key in is_general:
        user_key = re.sub('_brand', '_user', key)
        if key.endswith('_brand') and user_key in is_general:
            c += 1
            for counter, tid in enumerate(is_general_tids[key]):
                csvwriter.writerow([key[:-6], tid, is_general[user_key][counter], is_general[key][counter]])


import json

import requests
from bs4 import BeautifulSoup
from datetime import datetime
from ..requests_util import RequestsUtil


class ZillowCrawler:

    def __init__(self):
        self._p_sleep = 0.999
        self._t_sleep = 10
        self._input_fp = ''
        self._output_fp = ''
        self._hspace = 1

        self.requests = RequestsUtil()

    def set_input(self, input, **params):
        pass

    def set_output(self, output, **params):
        pass

    def set_progress_bar(self, hspace=1):
        self._hspace = int(hspace)
        return self

    def crawl(self):
        pass


def GetBoundingbox(zipcode):
    #call bing map
    url = 'http://dev.virtualearth.net/REST/v1/Locations/' \
        + zipcode + '/?o=json&key=AgYfRZaVa5_OqtLHE3v3hcMRKk3AhqstGb2SWmU6uMI4XpONOxNlES1UkBwoCRPP'
    try:
        r = requests.request('GET', url)
        parsed_json = json.loads(r.text)
        sw_lat = parsed_json['resourceSets'][0]['resources'][0]['bbox'][0]
        sw_long = parsed_json['resourceSets'][0]['resources'][0]['bbox'][1]
        ne_lat = parsed_json['resourceSets'][0]['resources'][0]['bbox'][2]
        ne_long = parsed_json['resourceSets'][0]['resources'][0]['bbox'][3]

        return ne_lat,ne_long,sw_lat,sw_long
    except Exception, e:
        print e


def GetZpidLisWithinZipcode(zipcode):

    listofproperties = []

    ne_lat,ne_long,sw_lat,sw_long = GetBoundingbox(zipcode)

    parts = str(ne_long).split('.')
    str_ne_long = parts[0] + parts[1][0:6]
    parts = str(sw_long).split('.')
    str_sw_long = parts[0] + parts[1][0:6]

    str_ne_lat = str(ne_lat).replace('.','')[0:8]
    str_sw_lat = str(sw_lat).replace('.','')[0:8]

    boundingbox = str_sw_long+','+str_sw_lat+','+str_ne_long+','+str_ne_lat

    url = 'http://www.zillow.com/search/GetResults.htm?spt=homes&status=110011&lt=111101&ht=111111&pr=,&mp=,&bd=0%2C&ba=0%2C&sf=,&lot=,&yr=,&pho=0&pets=0&parking=0&laundry=0&pnd=0&red=0&zso=0&days=any&ds=all&pmf=1&pf=1&zoom=10&rect=' + boundingbox +'&p=1&sort=days&search=map&disp=1&rt=7&listright=true&isMapSearch=true&zoom=10' #map vs. maplist
    # print url
    # url = 'http://www.zillow.com/homes/' + zipcode + '_rb'

    try:
        r = requests.request('GET', url)
        parsed_json = json.loads(r.text)

        # totalResultCount = parsed_json['list']['binCounts']['totalResultCount']
        # forSaleCount = parsed_json['list']['binCounts']['forSaleCount']
        # makeMeMoveCount = parsed_json['list']['binCounts']['makeMeMoveCount']
        # recentlySoldCount = parsed_json['list']['binCounts']['recentlySoldCount']
        # forRentCount = parsed_json['list']['binCounts']['forRentCount']
        # fsbaCount = parsed_json['list']['binCounts']['fsbaCount']
        # fsboCount = parsed_json['list']['binCounts']['fsboCount']
        # foreclosuresCount = parsed_json['list']['binCounts']['foreclosuresCount']
        # bankOwnedCount = parsed_json['list']['binCounts']['bankOwnedCount']
        # foreclosedCount = parsed_json['list']['binCounts']['foreclosedCount']
        # preForeclosureCount = parsed_json['list']['binCounts']['preForeclosureCount']
        # newConstructionCount = parsed_json['list']['binCounts']['newConstructionCount']
        # openHousesCount = parsed_json['list']['binCounts']['openHousesCount']
        # singleFamilyCount = parsed_json['list']['binCounts']['singleFamilyCount']
        # condoCount = parsed_json['list']['binCounts']['condoCount']
        # multiFamilyCount = parsed_json['list']['binCounts']['multiFamilyCount']
        # manufacturedCount = parsed_json['list']['binCounts']['manufacturedCount']
        # landCount = parsed_json['list']['binCounts']['landCount']
        # unmappedCount = parsed_json['list']['binCounts']['unmappedCount']
        # comingSoonCount = parsed_json['list']['binCounts']['comingSoonCount']

        properties = parsed_json['map']['properties']
        for property in properties:
            zpid = property[0]
            listofproperties.append(zpid)

        # print listofproperties
        return listofproperties

    except Exception, e:
        print e


def GetChart(zid, chartURL, chartType):
    path = basepath + zid + '_' + chartType + '.txt'
    with open(path, 'a') as fout:
        r = requests.request('GET', chartURL)
        parsed_json = json.loads(r.text)

        for entries in parsed_json['paparazziData']:
            data = entries['data']
            name = entries['name']
            regionType = entries['regionType']

            dslist = []
            valuelist = []

            for datapoint in data:
                xvalue = datapoint['xValue'] #time
                unixtime = int(xvalue/1000)
                ds = datetime.utcfromtimestamp(unixtime)
                yvalue = datapoint['yValue'] #value

                dslist.append(ds.strftime('%Y/%m/%d'))
                valuelist.append(yvalue)

        #print
            fout.writelines('\n'+name + '\t' + regionType + '\n')
            for i in range(len(dslist)):
                fout.writelines(dslist[i] + '\t' + str(valuelist[i]) + '\n')


def GetPropertyInfo(zpid):
    url = 'http://www.zillow.com/homedetails/' + zpid + '_zpid/'
    r = requests.request('GET', url)
    html = r.text
    # print html
    html = html.encode('ascii', 'ignore')
    content = BeautifulSoup(html)
    scriptTags = content.findAll('a', attrs={'data-chart-img': True})

    for script in scriptTags:
        chartURL = script['data-chart-img'].replace('render.png', 'render.json')

        if 'm=1' in chartURL:
            GetChart(zpid, chartURL,'zestimate')
        elif 'm=3' in chartURL:
            GetChart(zpid, chartURL,'listing_price')
        elif 'm=9' in chartURL:
            GetChart(zpid, chartURL,'rent_zestimate')
        elif 'm=2' in chartURL:
            GetChart(zpid, chartURL,'zestimate_chage')
        elif 'm=5' in chartURL:
            GetChart(zpid, chartURL,'tax_assessment')
        elif 'm=6' in chartURL:
            GetChart(zpid, chartURL,'tax_paid')
        elif 'm=4' in chartURL:
            GetChart(zpid, chartURL,'page_view')

#2098319300
basepath = '/Volumes/cchen224/'
GetPropertyInfo('2098319300')


import json
import time
import traceback
from datetime import datetime

import tweepy
from cutils import ProgressBar


# INPUT_FILE = '/Volumes/cchen224/sb50/f_broncos'
# OUTPUT_FILE = '/Volumes/cchen224/sb50/u_broncos.json'
# with open(OUTPUT_FILE, 'r') as i:
#     read_uids = set([json.loads(line)['id'] for line in i])
#
#
# with open(INPUT_FILE, 'r') as i:
#     target_uids = set([int(line.strip()) for line in i])


def get_user(uids, account):

    auth = tweepy.OAuthHandler(account['consumer_key'], account['consumer_secret'])
    auth.set_access_token(account['access_key'], account['access_secret'])
    api = tweepy.API(auth)
    return api.lookup_users(user_ids=uids)


accounts = [
    {'consumer_key': 'YT0tutGIiNJPN1qgeTbeycyZN',
     'consumer_secret': '3kr7n4wcpSKbJMg2HdUYuv7z6u8Jx5hYeaxEeDunEs94FxRpQI',
     'access_key': '954798157-kSCkePC2Nh6c7XNTDpGBrRW602EG2AAGlxBO742z',
     'access_secret': 'LSDcRb7uMlkzDfmwXlzY7puddMfdBfU8te5cWgbb4S7Rv'},

    {'consumer_key': 'jtsyzmLZh8PVeDFpDESNFhbfx',
     'consumer_secret': 'yf8qgkfrgOnqsLpOgIYRRjXNkWZ9RMSIvizW0MyfdcaWF73GVo',
     'access_key': '4175032576-IEdW79v1rKvaOgySkwlv6px4o7UFYshwACzdaKw',
     'access_secret': 'm6Fs8MzOflylebJoC82Zp8PsO7gAYhJ5TB9OYDULRHdih'},

    {'consumer_key': 'b4cbNQ34QnKrWZisexzevVQ7v',
     'consumer_secret': 'gFvkcmVUgt6BMWgx3MgdMgjuXa1ZAWceR00retenOVowJB2PVz',
     'access_key': '3965724493-Uxnvo0YX9XoqAmaD2L25iO4jgrlzldJas5GPMXM',
     'access_secret': 'GoRdhfiXsEuErQ6ktGDrmXm9x22EdootO9trET9MEUQtG'},

    {'consumer_key': 'Iee46Dnt3iAeFnCyPSPvgA49f',
     'consumer_secret': 'Vkth3Ocia9CGCyfOY1TISuDN7FHIfytvViRsV3dGnYfqfEPKTD',
     'access_key': '4007054717-30wMpW7IfV8gjp1orx3QO0bePy6TvFc6536JM80',
     'access_secret': 'zD55I7EgcmIUzzZ7fM7ueljtMSCnIRNBCQdaVvbzDU5Vu'},

    {'consumer_key': 'bxzC2N7lVo3Zq5M7osbSA',
     'consumer_secret': 'mSkxIXbnWrtMwhLRAenqHiQek1cOLGYWbtOqRiN1g',
     'access_key': '323394874-rzrfdsG6MhpjGtMwd8oLGB0ylpGtXCAEFAV8WAH1',
     'access_secret': 'hgccGUbMea8xvwDMlhi746CZA41KmnGQdoU1lWfCO0'},

    {'consumer_key': 'Nao9qC3oYqwGfaLXz2N58A',
     'consumer_secret': 'Mb4D9tLI2vOkfcoISQ9XmAankVvHMjgeU3tIca57fg',
     'access_key': '5765972-CABSQQQJ5dAz17bhnfqDkngK6VdtDO22lJf4Np4Wyl',
     'access_secret': '6DlGbyVpesCTndcQNzQPaAUo413zB3d4tpZKik'},

    {'consumer_key': '1Djhi40YzfrYF2Ff2VHIg',
     'consumer_secret': 'beeGpNxnY5e6jvDF3eoDgpPjXNFUjUmBcVPmt4HI',
     'access_key': '5765972-CBYl6MfD72eQGqfBpq0rcxaLNOMCWc4rtSbmfw3n7U',
     'access_secret': '6QdcrCuk3pUfDfAQHpi6K4WDrEo8JN3Avjkm0CkJv0M'},

    {'consumer_key': 'bh8eEkSjVSJ5prg8zQFpvA',
     'consumer_secret': 'lf5Vgf8TEb1JsEIsNQ4ywiIXmzf1qKHjkEMQ7xa8PA',
     'access_key': '5765972-KHZdV1U14btxXQEb0XSLGuxcU26lv6J2vl2IvUDBY',
     'access_secret': 'nbfGzDOt0tuy5DRu5brM7JeKziXstkkOdeJcFkRtsE'},

    {'consumer_key': 'HTxksPmuHyg0QAMZAliNmNBmF',
     'consumer_secret': 'ghvtEvXgHvMgyof33ouzTVzIaPgpRUH8L2vSGXv5yCvaCCgf17',
     'access_key': '323394874-0IbdxFKF6zcH9Pn55oqqV4PEa4RhZR3ayZ3qc3fP',
     'access_secret': 'HAtoaQTBzyIcWP4Btbcp0PMksU1fb17nzladnSU793RjU'},

    {'consumer_key': 'AI3wfoBCQxgewAfLF5yvygKhA',
     'consumer_secret': 'KBB6jcgFW835uq1B7hh4jJ4TxINeggTlMwpqqkGpFxoiksasrT',
     'access_key': '323394874-qEwwWxJrMTSN5KlY5reOeunFvYtVEf9zyFY6r9TP',
     'access_secret': 'TuulKdzw8Mdx9k877GXOjJ25fuQnPFHvcCtNXWgu0tZiq'},

    {'consumer_key': '7VwQ2daz3p7JRn5EZxP6or3KA',
     'consumer_secret': 'FhFASS0XA9exDFc2K74Jji7JT85dfMev7aV1aN1Hg5sLnJZuX2',
     'access_key': '323394874-6KBVniBhB7xdTQISX0vG0zwfc4BWYKhyXfngrQpR',
     'access_secret': 'd0UZtdmZtFlu6yWJyp3Hh2ewlMZ4txoh9CLLiq2gxh9sW'},

]


uids = list(target_uids - read_uids)
with open(OUTPUT_FILE, 'a') as o:
    c = 0
    e = 0
    a = 0
    bar = ProgressBar(total=len(uids) / 100 + 1)
    while True:
        if e >= 5:
            print c
            break
        try:
            user_ids = uids[c:(c+100)]
            if not user_ids:
                break
            try:
                users = get_user(user_ids, accounts[a])
            except:
                a += 1
                a %= len(accounts)
                users = get_user(user_ids, accounts[a])
            [o.write(json.dumps(user._json).encode('utf-8') + '\n') for user in users]
            bar.move().log()
            c += 100
            e = 0
        except Exception, error:
            e += 1
            time.sleep(5)
            print datetime.now().strftime('%Y-%m-%d %H:%M:%S'),
            print error



import re
from random import random
from time import sleep
import traceback
import requests
import json


def parse_text(js_tweet_text_container,
               is_remove_url=False,
               is_remove_mentions=False,
               is_remove_hashtags=False,
               is_remove_cashtags=False,
               is_remove_emojis=False,
               ):

    hashtags = []
    cashtags = []
    mentions = []
    emojis = dict()

    try:
        a_hrefs = js_tweet_text_container.p(['a', 'img'])

        for s in a_hrefs:
            if s.has_attr('data-expanded-url'):
                if is_remove_url:
                    s.extract()
            elif s.has_attr('data-query-source'):
                if s.attrs['data-query-source'] == 'cashtag_click':
                    cashtags.append(s.text.strip())
                    if is_remove_cashtags:
                        s.extract()
                elif s.attrs['data-query-source'] == 'hashtag_click':
                    hashtags.append(s.text.strip())
                    if is_remove_hashtags:
                        s.extract()

            elif s.has_attr('data-mentioned-user-id'):
                mentions.append(s.text.strip())
                if is_remove_mentions:
                    s.extract()
            elif s.has_attr('class') and 'Emoji--forText' in s.attrs['class']:
                emoji_alt = s.attrs['alt']
                emoji_title = '_'.join(s.attrs['title'].split())
                emojis[emoji_title] = emoji_alt
                if is_remove_emojis:
                    s.extract()
                else:
                    s.append(emoji_title)
    except TypeError:
        pass

    text = js_tweet_text_container.getText(separator=' ').encode('utf-8')
    text = re.sub('"|\n', ' ', text)
    text = re.sub(' +', ' ', text).strip()

    hashtags = '|'.join(hashtags).encode('utf-8')
    cashtags = '|'.join(cashtags).encode('utf-8')
    mentions = '|'.join(mentions).encode('utf-8')
    emojis = json.dumps(emojis) if emojis else ''

    return [text, hashtags, cashtags, mentions, emojis]



def parse_footer(stream_item_footer):
    # retweets = stream_item_footer.find('div', attrs={'title': 'Retweet'}).text
    retweets = stream_item_footer.find('span', class_=re.compile('retweet$')).text
    retweets = re.sub('[^0-9]', '', retweets)
    if retweets == '': retweets = u'0'

    # likes = stream_item_footer.find('div', attrs={'title': 'Like'}).text
    likes = stream_item_footer.find('span', class_=re.compile('favorite$')).text
    likes = re.sub('[^0-9]', '', likes)
    if likes == '': likes = u'0'


    return [retweets, likes]


def parse_fixer(js_tweet_details_fixer):
    # time = js_tweet_details_fixer.find('div', class_='client-and-actions').text.strip()
    stats = js_tweet_details_fixer.find('div', class_='js-tweet-stats-container tweet-stats-container ')
    try:
        retweets = stats.find('a', class_='request-retweeted-popup').text
        retweets = re.sub('[^0-9]', '', retweets)
    except AttributeError:
        retweets = '0'

    try:
        likes = stats.find('a', class_='request-favorited-popup').text
        likes = re.sub('[^0-9]', '', likes)
    except AttributeError:
        likes = '0'

    return [retweets, likes]


def parse_header(header_soup):
    timestamp = header_soup.find('span', class_=re.compile('timestamp$')).attrs.get('data-time')
    return [timestamp]


def parse_properties(soup):
    try:
        tweet_wrap = soup.find('div', class_=re.compile('js-stream-tweet|permalink-tweet'))
    except:
        return [False, False, False, False]
    properties = tweet_wrap.attrs
    uid = properties.get('data-user-id')
    tid = properties.get('data-item-id')
    screen_name = properties.get('data-screen-name')
    rid = properties.get('data-retweet-id', '')
    return [uid, screen_name, tid, rid]


def parse_quotetweet(quotetweet_container):
    try:
        properties = quotetweet_container.find('div', class_=re.compile('QuoteTweet-innerContainer')).attrs
        screen_name = properties.get('data-screen-name')
        tid = properties.get('data-item-id')
        text = quotetweet_container.find('div', class_='tweet-content').get_text(separator=' ', strip=True).encode('utf-8')
        return [screen_name, tid, text]
    except:
        return ['', '', '']


def get_html(url, n_retry=5):
    for i in xrange(n_retry):
        try:
            html = requests.request('GET', url).text
            return html
        except requests.ConnectionError:
            sleep(random() * 60)
            print 'Connection error', i,
        except requests.TooManyRedirects:
            sleep(60 * random())
            return None
    return None


# import json
# with open('/Volumes/cchen224/sb50/u_broncos.json', 'r') as i:
#     users = [json.loads(line)['screen_name'] for line in i]
# with open('/Volumes/cchen224/sb50/u_broncos_scn', 'w') as o:
#     [o.write(user + '\n') for user in users]


import csv
import json

BRANDS = ['Skittles',
          'Bud Light',
          'Advil',
          'Quicken',
          'Loans',
          'Acura',
          'Shock Top',
          'Taco Bell',
          'Marmot',
          'Squarespace',
          'Mountain Dew',
          'Audi',
          'Mobile Strike',
          'Hyundai',
          'PayPal',
          'Doritos',
          'SoFi',
          'Apartments',
          'Avocados From Mexico',
          'Avocados',
          'Snickers',
          'Michelob Ultra',
          'Pepsi',
          'Persil Proclean',
          'Toyota',
          'Coca-Cola',
          'OICisDifferent',
          'WeatherTech',
          'Dollar Shave Club',
          'T-Mobile',
          'TMobile',
          'Hyundai',
          'Buick',
          'Budweiser',
          'Honda',
          'Heinz',
          'Wix',
          'Fitbit',
          'Death Wish Coffee',
          'LG',
          'MINI USA',
          'MINI',
          'Pokemon',
          'Doritos',
          'TurboTax',
          'Jeep',
          'Amazon',
          'SunTrust',
          'Colgate',
          'Kia',
          'Axe',
          'Schick',
          '#StartStunning',
          '#Pokemon20',
          '#EveryDropCounts',
          '#GameDayStains',
          '#BaldwinBowl',
          '#MovinOnUp',
          '#Ballogize',
          '#GiveADamn',
          '#NotBackingDown',
          '#BestBuds',
          '#pepsihalftime',
          '#puppymonkeybaby',
          '#AvosInSpace',
          '#BolderThanBold',
          'MeetTheKetchups',
          '#DeclarationofDelicious',
          '#HyundaiSuperBowl',
          '#DefyLabels',
          '#AddPizzazz',
          '#GoPriusGo',
          '#TeamSmallBiz',
          '#onUp',
          '#SoFiGreat',
          '#RocketMortgage',
          '#EsuranceSweepstakes',
          '#CokeMini',
          'Jublia',
          'Mermot',
          'Machine Zone',
          'OIC',
          'Intel',
          'Bai',
          'Butterfinger',
          'Jack in the Box',
          'McDonalds',
          'Chrysler',
          'Intuit',
          'Esurance',
          'Quicken Loans',
          ]
BRANDS = list(set(BRANDS))

def get_follow_list(fp_in):
    with open(fp_in, 'r') as i:
        p = [line.strip() for line in i]
    return frozenset(p)


def merge_by_user(fp_in, fp_out, user_lst, **follower_list):
    import csv, json
    from datetime import datetime
    from datetime import timedelta
    from canalysis import get_sentiment
    from glob import glob
    import re

    t1 = datetime.strptime('2016-02-07 15:30:00', '%Y-%m-%d %H:%M:%S')
    t2 = datetime.strptime('2016-02-07 19:20:00', '%Y-%m-%d %H:%M:%S')


    users = dict()

    for file in glob(fp_in):
        with open(file, 'r') as i:
            if fp_in.endswith('.csv'):
                csvreader = csv.reader(i)
                for row in csvreader:
                    uid = row[0]
                    hashtags = set(row[5].split('|'))
                    emojis = row[8]
                    emojis = set(json.loads(row[8]).keys()) if emojis else set()

                    if uid not in users:
                        users[uid] = dict()
                        users[uid]['txt'] = row[4]
                        users[uid]['hst'] = hashtags
                        users[uid]['emj'] = emojis
                    else:
                        users[uid]['txt'] += ' ' + row[4]
                        users[uid]['hst'] |= hashtags
                        users[uid]['emj'] |= emojis

            elif fp_in.endswith('.json'):
                for line in i:
                    try:
                        jsontext = json.loads(line)
                        user = jsontext['user']
                    except:
                        continue

                    # tms = datetime.utcfromtimestamp(int(jsontext['timestamp_ms']) / 1000) - timedelta(hours=8)
                    # if tms > t2 or tms < t1:
                    #     continue

                    txt = jsontext['text'].encode('utf-8')
                    # if not re.compile('|'.join(BRANDS).lower()).search(txt.lower()):
                    #     continue

                    tid = jsontext['id_str']
                    uid = user['id_str']
                    scn = user['screen_name']

                    if uid not in user_lst:
                        continue

                    if uid in users:
                        users[uid]['txt'] += ' ' + txt
                        users[uid]['tid_count'] += 1
                    else:
                        users[uid] = dict()
                        users[uid]['txt'] = txt
                        users[uid]['tid_count'] = 1

    with open(fp_out, 'w') as o:
        csvwriter = csv.writer(o)
        for uid in users:
            follow = ''
            for team in follower_list:
                if uid in follower_list[team]:
                    follow = team
                    break
            users[uid]['follow'] = follow
            csvwriter.writerow([uid, follow, users[uid]['txt']])
    return users

p = get_follow_list('/Users/cchen224/Downloads/followers_panthers.txt')
b = get_follow_list('/Users/cchen224/Downloads/followers_broncos.txt')
pp = p - b
bb = b - p
bp = b & p
# merge_by_user('/Volumes/cchen224/sb50users.csv', '/Volumes/cchen224/sb50users_per_user.csv', broncos=bb, panthers=pp, both=bp)
# l = merge_by_user('/Volumes/cchen224/original/*.json', '/Volumes/cchen224/tweets_07_user_in_game.csv', user_list_ingame,broncos=bb, panthers=pp, both=bp)


def read_csv(fp_in, **y_label):
    import sys
    csv.field_size_limit(sys.maxsize)
    label = []
    text = []
    with open(fp_in, 'r') as i:
        csvreader = csv.reader(i)
        for row in csvreader:
            label.append(row[0])
            text.append(row[1])
    if y_label:
        label = [y_label[item] for item in label]
    return text, label

text, label = read_csv('/Volumes/cchen224/sb50_per_user_labeled.csv', both=0, broncos=1, panthers=2)


# from canalysis.ml_utilities import *
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.feature_extraction.text import TfidfTransformer
from sklearn.naive_bayes import MultinomialNB
from sklearn.pipeline import Pipeline
from sklearn.grid_search import GridSearchCV
from sklearn.linear_model import LogisticRegression, LassoCV
from sklearn.ensemble import GradientBoostingClassifier
from sklearn.cross_validation import cross_val_score
from sklearn.svm import SVC
from sklearn.pipeline import FeatureUnion
from sklearn.externals import joblib
from datetime import datetime
from sklearn.feature_selection import SelectKBest, SelectFdr, chi2, SelectFromModel
import numpy as np


model = Pipeline([
    ('count', CountVectorizer(stop_words='english')),
    ('tfidf', TfidfTransformer()),
    # ('mnnbc', MultinomialNB()),
    ('select', SelectKBest(SelectFdr, k=1e5)),
    ('boosting', GradientBoostingClassifier(n_estimators=10000, learning_rate=1.0, max_depth=5, random_state=0)),
    # ('logit', LogisticRegression(multi_class='multinomial',solver='lbfgs'))
])
parameters = {'count__ngram_range': [(1, 1), (1, 2), (1,3)],
              'tfidf__use_idf': (True, False),
              # 'mnnbc__alpha': (1e-2, 1e-3, 1e-5, 1e-6),
              'boosting__max_depth': [1,2,3,4,5],
              }
# print 'Start', datetime.now().strftime('%H:%M:%S')
# gs_clf = GridSearchCV(model, parameters, n_jobs=-1, cv=10)
# gs_clf = gs_clf.fit(text, label)
# print gs_clf.best_score_
# print gs_clf.best_params_, '\n\n'
# print gs_clf.grid_scores_
# print datetime.now().strftime('%H:%M:%S')


model_logistic = Pipeline([
    ('count', CountVectorizer(stop_words='english', ngram_range=(1,2), min_df=2)),
    ('tfidf', TfidfTransformer(use_idf=True)),
    # ('select', SelectKBest(chi2, k=1e5)),
    ('logit', LogisticRegression(multi_class='multinomial',solver='lbfgs'))
])
scores = cross_val_score(model_logistic, text, label, cv=10)
model_logistic = model_logistic.fit(text, label)


# joblib.dump(gs_clf, '/Volumes/cchen224/boost.pkl', compress=9)
# with open('/Volumes/cchen224/tweets_07_in_game_preference.tsv', 'r') as i:
#     user_list_ingame = set([line.strip().split('\t')[0] for line in i])

def get_follower(fp_in, fp_out, model):
    users_pref = dict()
    uids = []
    txts = []
    switch_v = {0: 'both', 1: 'broncos', 2: 'panthers'}
    with open(fp_in, 'r') as i:
        csvreader = csv.reader(i)
        for row in csvreader:
            if row[1]:
                users_pref[row[0]] = row[1]
            else:
                # users_pref[row[0]] = switch_v[model.predict([row[2]])[0]] + '*'
                uids.append(row[0])
                txts.append(row[2])

    print 'Starting prediction'
    pred = [switch_v[item] + '*' for item in model.predict(txts)]
    users_pref.update(dict(zip(uids, pred)))

    print 'Starting output'
    with open(fp_out, 'w') as o:
        for uid in users_pref:
            o.write('\t'.join([uid, users_pref[uid]]) + '\n')
    return users_pref
user_pref = get_follower('/Volumes/cchen224/tweets_07_user_in_game.csv',
                         '/Volumes/cchen224/tweets_07_in_game_preference.tsv',
                         model_logistic)

with open('/Volumes/cchen224/tweets_07_in_game_preference.tsv', 'r') as i:
    user_pref = dict(map(lambda (uid, follow): (uid, follow), [uf.strip().split('\t')[0:2] for uf in i]))

def get_raw_csv(fp_in, fp_out, **follower_list):
    import csv, json
    from datetime import datetime
    from datetime import timedelta
    from canalysis import get_sentiment
    import re



def get_sentiment_wrt_game(fp_in, fp_out, cutoff, preference):
    import csv, json
    from datetime import datetime
    from datetime import timedelta
    from canalysis import get_sentiment
    from vaderSentiment.vaderSentiment import sentiment as vaderSentiment
    import re
    from glob import glob

    t1 = datetime.strptime('2016-02-07 15:30:00', '%Y-%m-%d %H:%M:%S')
    t2 = datetime.strptime('2016-02-07 19:20:00', '%Y-%m-%d %H:%M:%S')

    timeline_l = [datetime.strptime('2016/01/01 15:30', '%Y/%m/%d %H:%M'),
                  datetime.strptime('2016/02/07 15:44', '%Y/%m/%d %H:%M'),
                  datetime.strptime('2016/02/07 16:07', '%Y/%m/%d %H:%M'),
                  datetime.strptime('2016/02/07 16:26', '%Y/%m/%d %H:%M'),
                  datetime.strptime('2016/02/07 16:31', '%Y/%m/%d %H:%M'),
                  datetime.strptime('2016/02/07 17:45', '%Y/%m/%d %H:%M'),
                  datetime.strptime('2016/02/07 17:51', '%Y/%m/%d %H:%M'),
                  datetime.strptime('2016/02/07 18:00', '%Y/%m/%d %H:%M'),
                  datetime.strptime('2016/02/07 18:08', '%Y/%m/%d %H:%M'),
                  datetime.strptime('2016/02/07 18:33', '%Y/%m/%d %H:%M'),
                  datetime.strptime('2016/02/07 18:43', '%Y/%m/%d %H:%M'),
                  datetime.strptime('2016/02/07 19:05', '%Y/%m/%d %H:%M'),
                  datetime.strptime('2016/02/07 19:08', '%Y/%m/%d %H:%M'),
                  datetime.strptime('2016/05/07 19:08', '%Y/%m/%d %H:%M')]
    timeline_d_den = {(timeline_l[0], timeline_l[1]) : '1',
                      (timeline_l[1], timeline_l[3]) : '1.23',
                      (timeline_l[3], timeline_l[4]) : '-0.5',
                      (timeline_l[4], timeline_l[5]) : '0.44',
                      (timeline_l[5], timeline_l[9]) : '0.83',
                      (timeline_l[9], timeline_l[-1]) : '2'}
    timeline_d_car = {(timeline_l[0], timeline_l[1]) : '1',
                      (timeline_l[1], timeline_l[3]) : '-0.73',
                      (timeline_l[3], timeline_l[4]) : '1',
                      (timeline_l[4], timeline_l[5]) : '-0.44',
                      (timeline_l[5], timeline_l[9]) : '-0.83',
                      (timeline_l[9], timeline_l[-1]) : '-2'}

    switch = {'both': 0, 'broncos': 1, 'panthers': 2}
    switch_v = {0: 'both', 1: 'broncos', 2: 'panthers'}

    facial = re.sub('(\#|\$|\(|\)|\[|\]|\?|\:|\*|\-|\\\\|\>|\<|\{|\})', '\ \\1', '|'.join(face.keys()))
    facial = re.sub(' ', '', facial)
    pattern10 = re.compile(r'(' + facial + r')')
    pattern11 = re.compile(r'(' + '|'.join(emoji_sent.keys()) + r')')
    pattern2 = re.compile(r'\b(' + '|'.join(illformed.keys()) + r')\b')
    pattern3 = re.compile('|'.join(BRANDS).lower())


    with open(fp_out, 'w') as o:
        csvwriter = csv.writer(o)
        csvwriter.writerow(['user_id','screen_name','tweet_id','timestamp','text','mentioned_brands','follow','sentiment_wrt_text','sentiment_wrt_game'])

        for file in glob(fp_in):
            with open(file, 'r') as i:
                if fp_in.endswith('.json'):
                    for line in i:
                        try:
                            jsontext = json.loads(line)
                            user = jsontext['user']
                        except:
                            continue

                        tms = datetime.utcfromtimestamp(int(jsontext['timestamp_ms']) / 1000) - timedelta(hours=8)
                        if tms > t2 or tms < t1:
                            # print tms.strftime('%H:%M:%S')
                            continue

                        txt = re.sub('\n' , ' ' , jsontext['text'])
                        txt = re.sub(' +', ' ', txt)
                        txt = pattern2.sub(lambda x: illformed[x.group()], txt)
                        txt = re.sub('http.+$', '', txt)

                        if pattern3.search(txt.lower()):
                            ads = '|'.join(pattern3.findall(txt.lower()))
                        else:
                            continue

                        tid = jsontext['id_str']
                        uid = user['id_str']
                        scn = user['screen_name']

                        snt = vaderSentiment(txt.encode('utf-8'))['compound']

                        txt_e = pattern11.sub(lambda x: str([[emoji_sent.get(x.group())]]), txt)
                        txt_e = pattern10.sub(lambda x: str([[face.get(x.group())]]), txt_e)
                        es = [float(item) for item in re.findall("\[\[([0-9.]*?)\]\]", txt_e) if item != 'None']
                        snt_e = sum(es) + snt


                        if snt_e < -1 * cutoff:
                            snt_e = 'neg'
                        elif snt_e > cutoff:
                            snt_e = 'pos'
                        else:
                            snt_e = 'neu'

                        follow = preference.get(uid, '')
                        if not follow:
                            print 'uid not found in preference.'
                            continue

                        sent_game = ''
                        for interval in timeline_d_den:
                            if interval[0] <= tms < interval[1]:
                                if re.sub('\*', '', follow) == 'broncos':
                                    sent_game = timeline_d_den[interval]
                                elif re.sub('\*', '', follow) == 'panthers':
                                    sent_game = timeline_d_car[interval]
                                else:
                                    sent_game = '0'
                        csvwriter.writerow([uid, scn, tid, tms.strftime('%Y-%m-%d %H:%M:%S'), txt.encode('utf-8'), ads, follow, snt_e, sent_game])
        # csvreader = csv.reader(i)
        # for row in csvreader:
get_sentiment_wrt_game('/Volumes/cchen224/original/tweets_0[6-7].json',
                       '/Volumes/cchen224/tweets_in_game_sentiment_update5.0.csv',
                       0.1,
                       user_pref)

with open('/Users/cchen224/Downloads/followers_panthers.txt', 'r') as i:
    p = [line.strip() for line in i]
    p = frozenset(p)

with open('/Users/cchen224/Downloads/followers_broncos.txt', 'r') as i:
    b = [line.strip() for line in i]
    b = frozenset(b)


pp_ = p - b
bb_ = b - p
bp_ = b & p






#############
from sklearn.cross_validation import KFold
for train_index, test_index in KFold(n=len(text), n_folds=10, shuffle=True, random_state=0):
    X_train, X_test = np.array(text)[train_index], np.array(text)[test_index]
    y_train, y_test = np.array(label)[train_index], np.array(label)[test_index]
    break

model = Pipeline([
    ('count', CountVectorizer(stop_words='english', ngram_range=(1,2), min_df=2)),
    ('tfidf', TfidfTransformer()),
    # ('mnnbc', MultinomialNB()),
    # ('select', SelectKBest(SelectFdr, k=1e5)),
    # ('boosting', GradientBoostingClassifier(n_estimators=10000, learning_rate=1.0, max_depth=5, random_state=0)),
    # ('logit', LogisticRegression(multi_class='multinomial',solver='lbfgs'))
])
X_train = model.fit_transform(X_train)
X_test = model.fit_transform(X_test)

# X_train = DenseTransformer.fit_transform(X_train)
# X_test = DenseTransformer.fit_transform(X_test)

boosting = GradientBoostingClassifier(n_estimators=10000, learning_rate=1.0, max_depth=5, random_state=0)
boosting = boosting.fit(X_train, y_train)


with open('/Users/cchen224/Downloads/data/corpus.v1.2.tweet', 'r') as i:
    illformed = dict()
    csvreader = csv.reader(i, delimiter='\t')
    for row in csvreader:
        if len(row) == 3 and row[1] == 'OOV':
            illformed[row[0]] = row[2]




cutoff = 0.1
with open('/Volumes/cchen224/tweets_07_sentiment_text_only.csv', 'w') as o:
    import csv, json
    from datetime import datetime
    from datetime import timedelta
    from canalysis import get_sentiment
    import re
    import emoji
    from vaderSentiment.vaderSentiment import sentiment as vaderSentiment
    csvwriter = csv.writer(o)
    facial = re.sub('(\#|\$|\(|\)|\[|\]|\?|\:|\*|\-|\\\\|\>|\<|\{|\})', '\ \\1', '|'.join(emoji_sent.keys()))
    facial = re.sub(' ', '', facial)
    pattern1 = re.compile(r'(' + facial + r')')
    pattern2 = re.compile(r'\b(' + '|'.join(illformed.keys()) + r')\b')


    with open('/Volumes/cchen224/tweets_07.json', 'r') as i:


        t1 = datetime.strptime('2016-02-07 15:30:00', '%Y-%m-%d %H:%M:%S')
        t2 = datetime.strptime('2016-02-07 19:20:00', '%Y-%m-%d %H:%M:%S')
        for line in i:
            try:
                jsontext = json.loads(line)
                user = jsontext['user']
            except:
                continue

            tms = datetime.utcfromtimestamp(int(jsontext['timestamp_ms']) / 1000) - timedelta(hours=8)
            if tms > t2 or tms < t1:
                continue

            txt = re.sub('\n' , ' ' , jsontext['text'])
            txt = pattern2.sub(lambda x: illformed[x.group()], txt)
            txt = re.sub('http.+$', '', txt)

            snt = vaderSentiment(txt.encode('utf-8'))['compound']

            txt_e = pattern1.sub(lambda x: str([[emoji_sent.get(x.group())]]), txt)
            es = [float(item) for item in re.findall("\[\[([0-9.]*?)\]\]", txt_e) if item != 'None']
            snt_e = sum(es) + snt

            if snt < -1 * cutoff:
                snt = 'neg'
            elif snt > cutoff:
                snt = 'pos'
            else:
                snt = 'neu'

            if snt_e < -1 * cutoff:
                snt_e = 'neg'
            elif snt_e > cutoff:
                snt_e = 'pos'
            else:
                snt_e = 'neu'
            csvwriter.writerow([snt, snt_e, txt.encode('utf-8')])


import math
def normalize(score, alpha=15):
    # normalize the score to be between -1 and 1 using an alpha that approximates the max expected value
    normScore = score/math.sqrt( ((score*score) + alpha) )
    return normScore


with open('/Volumes/cchen224/tweets_07_in_game_preference.tsv', 'r') as i:
    csvreader = csv.reader(i, delimiter='\t')
    users = set()
    tweets = set()
    for row in csvreader:
        users.add(row[0])
        tweets.add(row[1])
len(users)


def get_adjs(text):
    import nltk
    nltk_splitter = nltk.data.load('tokenizers/punkt/english.pickle')
    nltk_tokenizer = nltk.tokenize.TreebankWordTokenizer()
    sentences = nltk_splitter.tokenize(text)
    tokenized_sentences = [nltk_tokenizer.tokenize(sent) for sent in sentences]
    pos = [nltk.pos_tag(sentence) for sentence in tokenized_sentences]
    jjs = [' '.join([word[0] for word in words if 'JJ' in word[1]]) for words in pos]
    return ' '.join(jjs)


with open('/Volumes/cchen224/tweets_in_game_user.csv', 'w') as o:
    for file in glob('/Volumes/cchen224/p/*.csv'):
        with open(file, 'r') as i:
            [o.write(line.strip() + '\n') for line in i]

from datetime import datetime
timeline_b_ = [datetime.strptime('2016/01/01 15:30', '%Y/%m/%d %H:%M'),
               datetime.strptime('2016/02/07 16:26', '%Y/%m/%d %H:%M'),
               datetime.strptime('2016/02/07 17:33', '%Y/%m/%d %H:%M'),
               datetime.strptime('2016/02/07 17:45', '%Y/%m/%d %H:%M'),
               datetime.strptime('2016/02/07 18:33', '%Y/%m/%d %H:%M'),
               datetime.strptime('2016/02/07 19:21', '%Y/%m/%d %H:%M')]
timeline_b = {(timeline_b_[0], timeline_b_[1]) : 1,
              (timeline_b_[1], timeline_b_[2]) : 2,
              (timeline_b_[2], timeline_b_[3]) : 2.5,
              (timeline_b_[3], timeline_b_[4]) : 3,
              (timeline_b_[4], timeline_b_[5]) : 4,}


from glob import glob
import re
import csv
with open('/Volumes/cchen224/tweets_in_game_users.csv', 'r') as i:
    csvreader = csv.reader(i)
    users = dict(map(lambda (user_id, is_verified, bio, location, join_date, tweets, following, followers, likes):
                     (user_id, [is_verified, bio, location, join_date, tweets, following, followers, likes]),
                     (row for row in csvreader)))
with open('/Volumes/cchen224/tweets_in_game_hashtags.csv', 'r') as i:
    brands_hashtags = dict(map(lambda (x, y): (x.lower(), y), (line.strip().split(',') for line in i)))
with open('/Volumes/cchen224/tweets_in_game_ads_labels_update2.csv', 'r') as i:
    brands_emotions = dict()
    brands_appearance = dict()
    for line in i:
        if line.startswith('#'):
            continue
        tokens = line.strip().split(',')
        ad = tokens[0].lower()
        if ad not in brands_emotions:
            brands_emotions[ad] = [tokens[2:10]]
            brands_appearance[ad] = [2.5 if tokens[10] == 'halftime' else int(tokens[10])]
        else:
            brands_emotions[ad].append(tokens[2:10])
            brands_appearance[ad].append(2.5 if tokens[10] == 'halftime' else int(tokens[10]))

        if tokens[1]:
            ad = tokens[1].lower()
            if ad not in brands_emotions:
                brands_emotions[ad] = [tokens[2:10]]
                brands_appearance[ad] = [2.5 if tokens[10] == 'halftime' else int(tokens[10])]
            else:
                brands_emotions[ad].append(tokens[2:10])
                brands_appearance[ad].append(2.5 if tokens[10] == 'halftime' else int(tokens[10]))

with open('/Volumes/cchen224/tweets_in_game_sentiment_update5.0.csv', 'r') as i:
    csvreader = csv.reader(i)
    with open('/Volumes/cchen224/tweets_in_game_sentiment_update5.1.csv', 'w') as o:
        csvwriter = csv.writer(o)
        csvwriter.writerow(['#user_id', 'screen_name', 'tweet_id', 'timestamp', 'text', 'mentioned_brands',
                            'follow', 'sentiment_wrt_text', 'sentiment_wrt_game',
                            'is_verified', 'bio', 'location', 'join_date',
                            'n_tweets', 'n_following', 'n_followers', 'n_likes',
                            'apprearence (which quarter)',
                            'anger','fear','anticipation','surprise','joy','sadness','trust','disgust',
                            'ad_pos', 'ad_neg'])
        for row in csvreader:
            notfound = False
            quarter = None
            if row[0].startswith('#'):
                continue
            uinfo = users.get(row[1])
            if not uinfo: continue
            row.extend(uinfo)  # , ['', '', '', '', '', '', '', '']
            # brand_hashtag = [brands_appearance.get(brand, '') for brand in row[5].split('|') if '#' in brand]
            brand_hashtags = [brand for brand in row[5].split('|') if '#' in brand]
            try:
                brand_hashtag = brand_hashtags[0]
            except:
                brand_hashtag = brand_hashtags
                pass
            if brand_hashtag:
                emotions = ''
                for brand_hashtag in brand_hashtags:
                    emotions = brands_emotions.get(brand_hashtag, '')
                    if emotions:
                        break

                if not emotions:
                    if not [b for b in row[5].split('|') if not b.startswith('#')]:
                        notfound = True
                        continue

                    brand_emotion = ['','','','','','','','',]
                elif len(emotions) == 1:
                    quarter = brands_appearance[brand_hashtag][0]
                    brand_emotion = emotions[0]
                else:
                    tms = datetime.strptime(row[3], '%Y-%m-%d %H:%M:%S')
                    for time_interval in timeline_b:
                        if time_interval[0] <= tms < time_interval[1]:
                            quarter = timeline_b[time_interval]
                            break
                    if quarter not in brands_appearance[brand_hashtag]:
                        appearance = brands_appearance[brand_hashtag]
                        quarter = max(appearance) if quarter > max(appearance) else min(appearance)
                    brand_emotion = emotions[brands_appearance[brand_hashtag].index(quarter)]

            brands = [brand for brand in row[5].split('|') if not brand.startswith('#')]
            if not notfound and brands and not quarter:  # 2016-02-07 15:30:00

                brand_hashtag = ''
                # brands = [brand for brand in row[5].split('|') if not brand.startswith('#')]
                for brand in brands:
                    brand_hashtag = brands_appearance.get(brand, '')
                    if brand_hashtag:
                        brand_hashtag = brand
                        break

                if not brand_hashtag:
                    quarter = 0
                    brand_emotion = ['','','','','','','','',]
                    continue


                elif len(brands_emotions[brand_hashtag]) == 1:
                    quarter = brands_appearance[brand_hashtag][0]
                    brand_emotion = brands_emotions[brand_hashtag][0]
                else:
                    tms = datetime.strptime(row[3], '%Y-%m-%d %H:%M:%S')
                    for time_interval in timeline_b:
                        if time_interval[0] <= tms < time_interval[1]:
                            quarter = timeline_b[time_interval]
                            break
                    if quarter not in brands_appearance[brand_hashtag]:
                        appearance = brands_appearance[brand_hashtag]
                        quarter = max(appearance) if quarter > max(appearance) else min(appearance)
                    brand_emotion = brands_emotions[brand_hashtag][brands_appearance[brand_hashtag].index(quarter)]
                        # quarter = min([min(appearance, quarter) for appearance in brands_appearance[brand_hashtag]])
                # brand_hashtag = min(float(brands_appearance(row[5].split('|')[0])), float(brand_hashtag))

            if quarter == 2.5:
                row.append('halftime')
            elif notfound:
                row.append(99)
            else:
                row.append(quarter)
            row.extend(brand_emotion)
            brand_emotion_pos = sum([int(item) for counter, item in enumerate(brand_emotion)
                                     if counter in [2,3,4,6] and item])
            brand_emotion_neg = sum([int(item) for counter, item in enumerate(brand_emotion)
                                     if counter in [0,1,5,7] and item])
            row.append(brand_emotion_pos)
            row.append(brand_emotion_neg)
            csvwriter.writerow(row)


import csv
with open('/Volumes/cchen224/tweets_in_game_sentiment_update7.1.csv', 'r') as i:
    csvreader = csv.reader(i)
    head = csvreader.next()
    lines = [line for line in csvreader]
header = head[:7]
header.extend(['sentiment_wrt_text_p', 'sentiment_wrt_text_0.03', 'sentiment_wrt_text_0.05', 'sentiment_wrt_text_0.1', 'sentiment_wrt_text_vader',
               'sentiment_wrt_game_p', 'sentiment_wrt_game', #'sentiment_wrt_game_old',
               'sentiment_wrt_game_by_quarter'])
header.extend(head[9:])

def get_sent(value, cutoff):
    value = float(value)
    if value > cutoff:
        return 'pos'
    elif value < cutoff * -1:
        return 'neg'
    else:
        return 'neu'

with open('/Volumes/cchen224/tweets_in_game_sentiment_update5.0.csv', 'r') as i:
    csvreader = csv.reader(i)
    tid_sent = dict()
    for row in csvreader:
        if row[0].startswith('#'): continue
        tid_sent[row[2]] = row[7]

with open('/Volumes/cchen224/tweets_in_game_sentiment_update4.csv', 'r') as i:
    csvreader = csv.reader(i)
    tid_sent_game_old = dict()
    for row in csvreader:
        if row[0].startswith('#'): continue
        tid_sent_game_old[row[2]] = row[8]

from datetime import datetime
import re
timeline_l = [datetime.strptime('2016/01/01 15:30', '%Y/%m/%d %H:%M'),
              datetime.strptime('2016/02/07 15:44', '%Y/%m/%d %H:%M'),
              datetime.strptime('2016/02/07 16:07', '%Y/%m/%d %H:%M'),
              datetime.strptime('2016/02/07 16:26', '%Y/%m/%d %H:%M'),
              datetime.strptime('2016/02/07 16:31', '%Y/%m/%d %H:%M'),
              datetime.strptime('2016/02/07 17:45', '%Y/%m/%d %H:%M'),
              datetime.strptime('2016/02/07 17:51', '%Y/%m/%d %H:%M'),
              datetime.strptime('2016/02/07 18:00', '%Y/%m/%d %H:%M'),
              datetime.strptime('2016/02/07 18:08', '%Y/%m/%d %H:%M'),
              datetime.strptime('2016/02/07 18:33', '%Y/%m/%d %H:%M'),
              datetime.strptime('2016/02/07 18:43', '%Y/%m/%d %H:%M'),
              datetime.strptime('2016/02/07 19:05', '%Y/%m/%d %H:%M'),
              datetime.strptime('2016/02/07 19:08', '%Y/%m/%d %H:%M'),
              datetime.strptime('2016/05/07 19:08', '%Y/%m/%d %H:%M')]
timeline_d_den = {(timeline_l[0], timeline_l[1]) : '1',
                  (timeline_l[1], timeline_l[3]) : '0.23',
                  (timeline_l[3], timeline_l[5]) : '0.3',
                  (timeline_l[5], timeline_l[9]) : '0.83',
                  (timeline_l[9], timeline_l[-1]) : '1.14'}
timeline_d_car = {(timeline_l[0], timeline_l[1]) : '1',
                  (timeline_l[1], timeline_l[3]) : '-0.23',
                  (timeline_l[3], timeline_l[5]) : '-0.2',
                  (timeline_l[5], timeline_l[9]) : '-0.83',
                  (timeline_l[9], timeline_l[-1]) : '-1'}


with open('/Volumes/cchen224/tweets_in_game_sentiment_update8.csv', 'w') as o:
    csvwriter = csv.writer(o)
    csvwriter.writerow(header)
    for row in lines:
        tms = datetime.strptime(row[3], '%Y-%m-%d %H:%M:%S')
        follow = row[6]
        for interval in timeline_d_den:
            if interval[0] <= tms < interval[1]:
                if re.sub('\*', '', follow) == 'broncos':
                    sent_game = timeline_d_den[interval]
                elif re.sub('\*', '', follow) == 'panthers':
                    sent_game = timeline_d_car[interval]
                else:
                    sent_game = '0'
        out = row[:7]
        out.extend([row[7], get_sent(row[7], 0.03), get_sent(row[7], 0.05), get_sent(row[7], 0.1), tid_sent[row[2]],
                   row[8], get_sent(row[8], 0.) if row[8] != '1' else 'neu',
                        get_sent(sent_game, 0.) if sent_game != '1' else 'neu'])
        out.extend(row[9:])
        # row.insert(8, get_sent(row[7], 0.1))
        # row.insert(10, get_sent(row[9], 0) if row[9] != '1' else 'neu')
        # row.insert(9, tid_sent[row[2]])
        # row.insert(11, tid_sent_game_old[row[2]])

        csvwriter.writerow(out)

with open('/Volumes/cchen224/tweets_in_game_sentiment_update6_i_vs_j.csv', 'w') as o:
    csvwriter = csv.writer(o)
    with open('/Volumes/cchen224/tweets_in_game_sentiment_update6_0.05.csv', 'r') as i:
        csvreader = csv.reader(i)
        csvreader.next()
        for row in csvreader:
            if row[8] != row[9]:
                if 'neu' not in [row[8], row[9]]:
                    csvwriter.writerow([row[2], row[4], row[8], row[9]])






from glob import glob
tweets = []
for file in glob('/Users/cchen224/Downloads/expended_thread_aws/*.csv.txt'):
    with open(file, 'r') as i:
        brand = re.sub('^@|[.]csv[.]txt$', '', file.split('/')[-1])
        csvreader = csv.reader(i, delimiter='\t')
        is_use = True
        for row in csvreader:
            if is_use:
                tweets.append([brand, row[0], row[1], re.sub('\n|\r', ' ', row[3])])
                is_use = False
            elif not row:
                is_use = True

tweets = [tweet for tweet in tweets if not re.compile(tweet[0].lower()).search(tweet[3].lower())]

with open('/Volumes/cchen224/amazon_aws_brands.csv', 'w') as o:
    csvwriter = csv.writer(o)
#
#
#
# import praw
# import webbrowser
# import re
# r = praw.Reddit(user_agent='ccTest')
# r.set_oauth_app_info(client_id='FdkfxBHQeKNfaQ',
#                      client_secret='7U1mGVNkuHfQ2nU21MeVSWA5xOs',
#                      redirect_uri='http://127.0.0.1:65010/authorize_callback')
# url = r.get_authorize_url('uniqueKey', 'read', True)
# webbrowser.open(url)
# access_information = r.get_access_information('VtiF9ev2b8qVsSs5UQmH4ubX294') # use token after "code =" in url
# r.set_access_credentials(**access_information)
# subreddit = r.get_subreddit('SuicideWatch')
# submissions = [submission for submission in subreddit.get_hot(limit=9999999)]
#
#
#
# import re
# import csv
#
#
# def parse_comments(comment):
#     title = comment.title if hasattr(comment, 'title') else ''
#     author_name = comment.author.name if comment.author else ''
#     author_id = comment.author.id if comment.author else ''
#     parent_id = comment.parent_id if hasattr(comment, 'parent_id') else ''
#     parent_id = re.sub('t[0-9]\\_', '', parent_id)
#     post_id = comment.id
#     post_text = comment.body.encode('utf-8') if hasattr(comment, 'body') else comment.selftext.encode('utf-8')
#     post_text = re.sub('\n|\r', ' ', post_text)
#     voteups = comment.ups
#     votedowns = comment.downs
#     timestamp = int(comment.created_utc)
#     out = [[title, author_name, author_id, post_id, parent_id, timestamp, post_text, voteups, votedowns, ]]
#
#     replies = comment.comments if hasattr(comment, 'comments') else comment.replies
#     if replies:
#         for reply in replies:
#             out.extend(parse_comments(reply))
#         return out
#     else:
#         return out
#
#
# with open('/Users/cchen224/Downloads/reddit.csv', 'w') as o:
#     csvwriter = csv.writer(o)
#     csvwriter.writerow(['title',
#                         'author_name', 'author_id', 'post_id', 'parent_id',
#                         'timestamp', 'post_text', 'voteups', 'votedowns',])
#     for submission in submissions:
#         [csvwriter.writerow(item) for item in parse_comments(submission)]



import requests


class RequestsUtil:

    def __init__(self, max_retries=10, header=None):
        self.session = requests.Session()
        adapter = requests.adapters.HTTPAdapter(max_retries=max_retries)
        self.session.mount('https://', adapter)
        self.header = header

    def get(self, url, timeout=60):
        return self.session.get(url, headers=self.header, timeout=timeout)



# import csv
# with open('/Volumes/cchen224/sb50/mappingback.csv', 'r') as i:
#     csvreader = csv.reader(i)
#     mapping = dict(map(lambda (x, y): (x.lower(), y.lower()), (row for row in csvreader)))
# with open('/Volumes/cchen224/sb50/tweets_in_game_sentiment_update6_0.1.csv', 'r') as i:
#     csvreader = csv.reader(i)
#     next(csvreader)
#     out = []
#     for row in csvreader:
#         brands = set()
#         for item in row[5].split('|'):
#             brands.add(mapping[item])
#         if 'pepsi' in brands and 'doritos' in brands:
#             brands.remove('pepsi')
#         if 'pepsi' in brands and 'mountain dew' in brands:
#             brands.remove('pepsi')
#         if '' in brands and len(brands) > 1:
#             brands.remove('')
#         if '' in brands and len(brands) == 1:
#             out.append('@_@')
#         else:
#             out.append('|'.join(brands))
#
# with open('/Volumes/cchen224/sb50/tweets_in_game_sentiment_update6_0.1_mentions.csv', 'w') as o:
#     [o.write(item + '\n') for item in out]


import time

import tweepy


def get_followers_ids(user_id, account):

    auth = tweepy.OAuthHandler(account['consumer_key'], account['consumer_secret'])
    auth.set_access_token(account['access_key'], account['access_secret'])
    api = tweepy.API(auth)

    # f = open(user_id + '_followings.txt', 'w+')
    current_page = 1
    user_timeline = tweepy.Cursor(api.user_timeline, user_id=user_id, since_id=696163960001679361, max_id=None, lang='en').pages()
    # user_timeline = tweepy.Cursor(api.user_timeline, screen_name=user_id, lang='en').pages()
    # print 'Starting collect ' + screen_name
    while True:
        try:
            print '\tGetting page ' + str(current_page)
            ids = user_timeline.next()
            # for i in ids:
                # f.write(str(i) + '\n')
            current_page += 1
            time.sleep(60)
        except tweepy.TweepError:
            print '\tWaiting on rate limit.'
            time.sleep(60)
        except StopIteration:
            # print screen_name + ' Done!\n\n\n'
            break
    f.close()


# import json
# f = open('/Users/cchen224/Downloads/tweets_07.json')
# file = json.load(f)
# 
#
#
# user_id=2634592777
#
# import time
# from tweepy.error import TweepError
# with open('/Users/cchen224/Downloads/brandlist.txt', 'r') as i:
#     users = [line.strip() for line in i]
# # geo = dict()
# # with open('/Users/cchen224/Downloads/brandlist.geo', 'w') as o:
#     for user_id in users:
#         if user_id not in geo:# and user_id not in ['suncelltweets', 'Mobilink', 'BeatsMusicHelp', 'ShawDirect_News']:
#             print user_id
#             try:
#                 geo[user_id] = tweepy.Cursor(api.user_timeline, screen_name=user_id, lang='en').pages().next()[0]._json['user']['time_zone']
#                 # time.sleep(5)
#             except TweepError, e:
#                 pass
#             except StopIteration:
#                 pass
#
# with open('/Users/cchen224/Downloads/brandlist.geo', 'w') as o:
#     [o.write('\t'.join([key, geo[key]]) + '\n') for key in geo if geo[key]]
#



__author__ = 'hyheng'
from bs4 import BeautifulSoup
# from bs4 import BeautifulSoup
from urllib2 import urlopen
import json
import requests
import re
import csv
from datetime import datetime
from time import gmtime, strftime
from pyparsing import Literal, quotedString, removeQuotes, delimitedList
import json


def ConvertUnixTime(unixtime):
    ds = datetime.fromtimestamp(unixtime)
    return ds

def GetBoundingbox(zipcode):
    #call bing map
    url = 'http://dev.virtualearth.net/REST/v1/Locations/' \
        + zipcode + '/?o=json&key=AgYfRZaVa5_OqtLHE3v3hcMRKk3AhqstGb2SWmU6uMI4XpONOxNlES1UkBwoCRPP'
    try:
        r = requests.request('GET', url)
        parsed_json = json.loads(r.text)
        sw_lat = parsed_json['resourceSets'][0]['resources'][0]['bbox'][0]
        sw_long = parsed_json['resourceSets'][0]['resources'][0]['bbox'][1]
        ne_lat = parsed_json['resourceSets'][0]['resources'][0]['bbox'][2]
        ne_long = parsed_json['resourceSets'][0]['resources'][0]['bbox'][3]

        return ne_lat,ne_long,sw_lat,sw_long
    except Exception, e:
        print e

def GetZpidLisWithinZipcode(zipcode):

    listofproperties = []

    ne_lat,ne_long,sw_lat,sw_long = GetBoundingbox(zipcode)

    parts = str(ne_long).split('.')
    str_ne_long = parts[0] + parts[1][0:6]
    parts = str(sw_long).split('.')
    str_sw_long = parts[0] + parts[1][0:6]

    str_ne_lat = str(ne_lat).replace('.','')[0:8]
    str_sw_lat = str(sw_lat).replace('.','')[0:8]

    boundingbox = str_sw_long+','+str_sw_lat+','+str_ne_long+','+str_ne_lat

    url = 'http://www.zillow.com/search/GetResults.htm?spt=homes&status=110011&lt=111101&ht=111111&pr=,&mp=,&bd=0%2C&ba=0%2C&sf=,&lot=,&yr=,&pho=0&pets=0&parking=0&laundry=0&pnd=0&red=0&zso=0&days=any&ds=all&pmf=1&pf=1&zoom=10&rect=' + boundingbox +'&p=1&sort=days&search=map&disp=1&rt=7&listright=true&isMapSearch=true&zoom=10' #map vs. maplist
    # print url
    # url = 'http://www.zillow.com/homes/' + zipcode + '_rb'

    try:
        r = requests.request('GET', url)
        parsed_json = json.loads(r.text)

        # totalResultCount = parsed_json['list']['binCounts']['totalResultCount']
        # forSaleCount = parsed_json['list']['binCounts']['forSaleCount']
        # makeMeMoveCount = parsed_json['list']['binCounts']['makeMeMoveCount']
        # recentlySoldCount = parsed_json['list']['binCounts']['recentlySoldCount']
        # forRentCount = parsed_json['list']['binCounts']['forRentCount']
        # fsbaCount = parsed_json['list']['binCounts']['fsbaCount']
        # fsboCount = parsed_json['list']['binCounts']['fsboCount']
        # foreclosuresCount = parsed_json['list']['binCounts']['foreclosuresCount']
        # bankOwnedCount = parsed_json['list']['binCounts']['bankOwnedCount']
        # foreclosedCount = parsed_json['list']['binCounts']['foreclosedCount']
        # preForeclosureCount = parsed_json['list']['binCounts']['preForeclosureCount']
        # newConstructionCount = parsed_json['list']['binCounts']['newConstructionCount']
        # openHousesCount = parsed_json['list']['binCounts']['openHousesCount']
        # singleFamilyCount = parsed_json['list']['binCounts']['singleFamilyCount']
        # condoCount = parsed_json['list']['binCounts']['condoCount']
        # multiFamilyCount = parsed_json['list']['binCounts']['multiFamilyCount']
        # manufacturedCount = parsed_json['list']['binCounts']['manufacturedCount']
        # landCount = parsed_json['list']['binCounts']['landCount']
        # unmappedCount = parsed_json['list']['binCounts']['unmappedCount']
        # comingSoonCount = parsed_json['list']['binCounts']['comingSoonCount']

        properties = parsed_json['map']['properties']
        for property in properties:
            zpid = property[0]
            listofproperties.append(zpid)

        # print listofproperties
        return listofproperties

    except Exception, e:
        print e

def GetChart(zid, chartURL, chartType):
    path = basepath + zid + '_' + chartType + '.txt'
    with open(path, 'a') as fout:
        r = requests.request('GET', chartURL)
        parsed_json = json.loads(r.text)

        for entries in parsed_json['paparazziData']:
            data = entries['data']
            name = entries['name']
            regionType = entries['regionType']

            dslist = []
            valuelist = []

            for datapoint in data:
                xvalue = datapoint['xValue'] #time
                unixtime = int(xvalue/1000)
                ds = ConvertUnixTime(unixtime)
                yvalue = datapoint['yValue'] #value

                dslist.append(ds.strftime('%Y/%m/%d'))
                valuelist.append(yvalue)

        #print
            fout.writelines('\n'+name + '\t' + regionType + '\n')
            for i in range(len(dslist)):
                fout.writelines(dslist[i] + '\t' + str(valuelist[i]) + '\n')


def GetPropertyInfo(zpid):
    url = 'http://www.zillow.com/homedetails/' + zpid + '_zpid/'
    r = requests.request('GET', url)
    html = r.text
    # print html
    html = html.encode('ascii', 'ignore')
    content = BeautifulSoup(html)
    scriptTags = content.findAll('a', attrs={'data-chart-img': True})

    for script in scriptTags:
        chartURL = script['data-chart-img'].replace('render.png','render.json')

        if 'm=1' in chartURL:
            GetChart(zpid, chartURL,'zestimate')
        elif 'm=3' in chartURL:
            GetChart(zpid, chartURL,'listing_price')
        elif 'm=9' in chartURL:
            GetChart(zpid, chartURL,'rent_zestimate')
        elif 'm=2' in chartURL:
            GetChart(zpid, chartURL,'zestimate_chage')
        elif 'm=5' in chartURL:
            GetChart(zpid, chartURL,'tax_assessment')
        elif 'm=6' in chartURL:
            GetChart(zpid, chartURL,'tax_paid')
        elif 'm=4' in chartURL:
            GetChart(zpid, chartURL,'page_view')





# GetPropertyInfo('19619228')

# GetZpidLisWithinZipcode('60687')

basepath = '/Users/hyheng/Dropbox/zillow-project/nyc msa/'

filein = '/Users/hyheng/Dropbox/zillow-project/nyc msa[property sample2].txt'
lines = [line.strip() for line in open(filein, 'r')]
index = 0
for i in range(index, len(lines)):
    parts = lines[i].split(',')
    print parts[0]
    try:
        for j in range(1, len(parts)):
            if len(parts[j]) > 1:
                GetPropertyInfo(parts[j])
    except Exception, e:
        print e


# filein = '/Users/hyheng/Dropbox/zillow-project/nyc msa.txt'
#
# lines = [line.strip() for line in open(filein, 'r')]
#
# filepath2 = '/Users/hyheng/Dropbox/zillow-project/nyc msa[property].txt'
#
# index = 0
#
# with open(filepath2, 'a') as fout:
#     for i in range(index, len(lines)):
#         zip = lines[i].split('\t')[0]
#         print zip
#         fout.write(zip+',')
#
#         try:
#             listofproperties = GetZpidLisWithinZipcode(zip)
#
#
#             for item in listofproperties:
#                 fout.write(str(item)+',')
#
#             fout.write('\n')
#
#         except Exception, e:
#             print e


"""
(C) Mathieu Blondel - 2010
License: BSD 3 clause
Implementation of the collapsed Gibbs sampler for
Latent Dirichlet Allocation, as described in
Finding scientifc topics (Griffiths and Steyvers)
"""

import numpy as np
import scipy as sp
from scipy.special import gammaln

def sample_index(p):
    """
    Sample from the Multinomial distribution and return the sample index.
    """
    return np.random.multinomial(1,p).argmax()

def word_indices(vec):
    """
    Turn a document vector of size vocab_size to a sequence
    of word indices. The word indices are between 0 and
    vocab_size-1. The sequence length is equal to the document length.
    """
    for idx in vec.nonzero()[0]:
        for i in xrange(int(vec[idx])):
            yield idx

def log_multi_beta(alpha, K=None):
    """
    Logarithm of the multinomial beta function.
    """
    if K is None:
        # alpha is assumed to be a vector
        return np.sum(gammaln(alpha)) - gammaln(np.sum(alpha))
    else:
        # alpha is assumed to be a scalar
        return K * gammaln(alpha) - gammaln(K*alpha)

class LdaSampler(object):

    def __init__(self, n_topics, alpha=0.1, beta=0.1):
        """
        n_topics: desired number of topics
        alpha: a scalar (FIXME: accept vector of size n_topics)
        beta: a scalar (FIME: accept vector of size vocab_size)
        """
        self.n_topics = n_topics
        self.alpha = alpha
        self.beta = beta

    def _initialize(self, matrix):
        n_docs, vocab_size = matrix.shape

        # number of times document m and topic z co-occur
        self.nmz = np.zeros((n_docs, self.n_topics))
        # number of times topic z and word w co-occur
        self.nzw = np.zeros((self.n_topics, vocab_size))
        self.nm = np.zeros(n_docs)
        self.nz = np.zeros(self.n_topics)
        self.topics = {}

        for m in xrange(n_docs):
            # i is a number between 0 and doc_length-1
            # w is a number between 0 and vocab_size-1
            for i, w in enumerate(word_indices(matrix[m, :])):
                # choose an arbitrary topic as first topic for word i
                z = np.random.randint(self.n_topics)
                self.nmz[m,z] += 1
                self.nm[m] += 1
                self.nzw[z,w] += 1
                self.nz[z] += 1
                self.topics[(m,i)] = z

    def _conditional_distribution(self, m, w):
        """
        Conditional distribution (vector of size n_topics).
        """
        vocab_size = self.nzw.shape[1]
        left = (self.nzw[:,w] + self.beta) / \
               (self.nz + self.beta * vocab_size)
        right = (self.nmz[m,:] + self.alpha) / \
                (self.nm[m] + self.alpha * self.n_topics)
        p_z = left * right
        # normalize to obtain probabilities
        p_z /= np.sum(p_z)
        return p_z

    def loglikelihood(self):
        """
        Compute the likelihood that the model generated the data.
        """
        vocab_size = self.nzw.shape[1]
        n_docs = self.nmz.shape[0]
        lik = 0

        for z in xrange(self.n_topics):
            lik += log_multi_beta(self.nzw[z,:]+self.beta)
            lik -= log_multi_beta(self.beta, vocab_size)

        for m in xrange(n_docs):
            lik += log_multi_beta(self.nmz[m,:]+self.alpha)
            lik -= log_multi_beta(self.alpha, self.n_topics)

        return lik

    def phi(self):
        """
        Compute phi = p(w|z).
        """
        V = self.nzw.shape[1]
        num = self.nzw + self.beta
        num /= np.sum(num, axis=1)[:, np.newaxis]
        return num

    def run(self, matrix, maxiter=30):
        """
        Run the Gibbs sampler.
        """
        n_docs, vocab_size = matrix.shape
        self._initialize(matrix)

        for it in xrange(maxiter):
            for m in xrange(n_docs):
                for i, w in enumerate(word_indices(matrix[m, :])):
                    z = self.topics[(m,i)]
                    self.nmz[m,z] -= 1
                    self.nm[m] -= 1
                    self.nzw[z,w] -= 1
                    self.nz[z] -= 1

                    p_z = self._conditional_distribution(m, w)
                    z = sample_index(p_z)

                    self.nmz[m,z] += 1
                    self.nm[m] += 1
                    self.nzw[z,w] += 1
                    self.nz[z] += 1
                    self.topics[(m,i)] = z

            # FIXME: burn-in and lag!
            yield self.phi()

if __name__ == "__main__":
    import os
    import shutil

    N_TOPICS = 10
    DOCUMENT_LENGTH = 100
    FOLDER = "topicimg"

    def vertical_topic(width, topic_index, document_length):
        """
        Generate a topic whose words form a vertical bar.
        """
        m = np.zeros((width, width))
        m[:, topic_index] = int(document_length / width)
        return m.flatten()

    def horizontal_topic(width, topic_index, document_length):
        """
        Generate a topic whose words form a horizontal bar.
        """
        m = np.zeros((width, width))
        m[topic_index, :] = int(document_length / width)
        return m.flatten()

    def save_document_image(filename, doc, zoom=2):
        """
        Save document as an image.
        doc must be a square matrix
        """
        height, width = doc.shape
        zoom = np.ones((width*zoom, width*zoom))
        # imsave scales pixels between 0 and 255 automatically
        sp.misc.imsave(filename, np.kron(doc, zoom))

    def gen_word_distribution(n_topics, document_length):
        """
        Generate a word distribution for each of the n_topics.
        """
        width = n_topics / 2
        vocab_size = width ** 2
        m = np.zeros((n_topics, vocab_size))

        for k in range(width):
            m[k,:] = vertical_topic(width, k, document_length)

        for k in range(width):
            m[k+width,:] = horizontal_topic(width, k, document_length)

        m /= m.sum(axis=1)[:, np.newaxis] # turn counts into probabilities

        return m

    def gen_document(word_dist, n_topics, vocab_size, length=DOCUMENT_LENGTH, alpha=0.1):
        """
        Generate a document:
            1) Sample topic proportions from the Dirichlet distribution.
            2) Sample a topic index from the Multinomial with the topic
               proportions from 1).
            3) Sample a word from the Multinomial corresponding to the topic
               index from 2).
            4) Go to 2) if need another word.
        """
        theta = np.random.dirichlet([alpha] * n_topics)
        v = np.zeros(vocab_size)
        print v
        for n in range(length):
            z = sample_index(theta)
            w = sample_index(word_dist[z,:])
            v[w] += 1
        return v

    def gen_documents(word_dist, n_topics, vocab_size, n=500):
        """
        Generate a document-term matrix.
        """
        m = np.zeros((n, vocab_size))
        for i in xrange(n):
            m[i, :] = gen_document(word_dist, n_topics, vocab_size)
        return m

    if os.path.exists(FOLDER):
        shutil.rmtree(FOLDER)
    os.mkdir(FOLDER)

    width = N_TOPICS / 2
    vocab_size = width ** 2
    word_dist = gen_word_distribution(N_TOPICS, DOCUMENT_LENGTH)
    matrix = gen_documents(word_dist, N_TOPICS, vocab_size)
    sampler = LdaSampler(N_TOPICS)

    for it, phi in enumerate(sampler.run(matrix)):
        print "Iteration", it
        print "Likelihood", sampler.loglikelihood()

        # if it % 5 == 0:
        #     for z in range(N_TOPICS):
        #         save_document_image("topicimg/topic%d-%d.png" % (it,z),
        #                             phi[z,:].reshape(width,-1))



# Author: Olivier Grisel <olivier.grisel@ensta.org>
#         Lars Buitinck <L.J.Buitinck@uva.nl>
#         Chyi-Kwei Yau <chyikwei.yau@gmail.com>
# License: BSD 3 clause

from __future__ import print_function
from time import time

from sklearn.feature_extraction.text import TfidfVectorizer, CountVectorizer
from sklearn.decomposition import NMF, LatentDirichletAllocation
from sklearn.datasets import fetch_20newsgroups

n_samples = 2000
n_features = 1000
n_topics = 10
n_top_words = 20


def print_top_words(model, feature_names, n_top_words):
    for topic_idx, topic in enumerate(model.components_):
        print("Topic #%d:" % topic_idx)
        print(" ".join([feature_names[i]
                        for i in topic.argsort()[:-n_top_words - 1:-1]]))
    print()


# Load the 20 newsgroups dataset and vectorize it. We use a few heuristics
# to filter out useless terms early on: the posts are stripped of headers,
# footers and quoted replies, and common English words, words occurring in
# only one document or in at least 95% of the documents are removed.

print("Loading dataset...")
t0 = time()
dataset = fetch_20newsgroups(shuffle=True, random_state=1,
                             remove=('headers', 'footers', 'quotes'))
data_samples = dataset.data
print("done in %0.3fs." % (time() - t0))

# Use tf-idf features for NMF.
print("Extracting tf-idf features for NMF...")
tfidf_vectorizer = TfidfVectorizer(max_df=0.95, min_df=2, #max_features=n_features,
                                   stop_words='english')
t0 = time()
tfidf = tfidf_vectorizer.fit_transform(data_samples)
print("done in %0.3fs." % (time() - t0))

# Use tf (raw term count) features for LDA.
print("Extracting tf features for LDA...")
tf_vectorizer = CountVectorizer(max_df=0.95, min_df=2, max_features=n_features,
                                stop_words='english')
t0 = time()
tf = tf_vectorizer.fit_transform(data_samples)
print("done in %0.3fs." % (time() - t0))

# Fit the NMF model
print("Fitting the NMF model with tf-idf features,"
      "n_samples=%d and n_features=%d..."
      % (n_samples, n_features))
t0 = time()
nmf = NMF(n_components=n_topics, random_state=1, alpha=.1, l1_ratio=.5).fit(tfidf)
exit()
print("done in %0.3fs." % (time() - t0))

print("\nTopics in NMF model:")
tfidf_feature_names = tfidf_vectorizer.get_feature_names()
print_top_words(nmf, tfidf_feature_names, n_top_words)

print("Fitting LDA models with tf features, n_samples=%d and n_features=%d..."
      % (n_samples, n_features))
lda = LatentDirichletAllocation(n_topics=n_topics, max_iter=5,
                                learning_method='online', learning_offset=50.,
                                random_state=0)
t0 = time()
lda.fit(tf)
print("done in %0.3fs." % (time() - t0))

print("\nTopics in LDA model:")
tf_feature_names = tf_vectorizer.get_feature_names()
print_top_words(lda, tf_feature_names, n_top_words)


from random import random
from datetime import datetime

from bs4 import BeautifulSoup
from time import sleep

from cutils import Gmail
from webparser_error import *
from webparser_utility import *


def parse_tweet(tiduid):
    """
    :param tiduid: tid@uid
    :return: uid, screen_name, tid, timestamp, text, hashtags, cashtags, mentions, emojis, retweets, likes
    """
    session = requests.Session()
    adapter = requests.adapters.HTTPAdapter(max_retries=10)
    session.mount('https://', adapter)

    tokens = tiduid.split('@')
    tid = tokens[0]
    uid = tokens[1]
    url = 'https://twitter.com/' + uid + '/status/' + tid
    res = session.get(url)
    html = res._content

    if html is None:
        return [[]]

    soup = BeautifulSoup(html, 'html.parser')
    error_functions = [is_not_account_suspended(soup),
                       is_not_content_protected(soup),
                       is_page_exists(soup),
                       is_not_tweet_withheld(soup)]
    for fnc in error_functions:
        if not fnc:
            return []

    if soup is None:
        # with open('/Users/cchen224/Downloads/web_error.html', 'w') as o:
        #     o.write(res.__str__() + '\n\n\n' + html)
        sleep(random()*10)
        # print 1
        # Gmail().set_from_to().set_credentials().set_subject('AWS Twitter.tweet PropertiesNotFound').send_message(tiduid + '\n\n' + soup.__str__()).close()
        return [[]]
    # PermalinkOverlay
    cardwrap = soup.find('div', class_='permalink-inner permalink-tweet-container')

    uid, screen_name, tid, rid = parse_properties(cardwrap)
    # try:
    #
    #     properties = cardwrap.find('div', class_=re.compile('^permalink-tweet')).attrs
    #     tid = properties['data-item-id']
    #     uid = properties['data-user-id']
    #     screen_name = properties['data-screen-name']
    # except TypeError:
    #     t = random() * 60
    #     print tiduid, 'sleeping', t, 'secs.', datetime.now().strftime('%Y/%m/%d %H:%M:%S')
    #     Gmail().set_from_to().set_credentials().set_subject('AWS Twitter.tweet PropertiesNotFound').send_message(tiduid + '\n\n' + html.__str__()).close()
    #     sleep(t)
    #     return parse_tweet(tiduid)

    if uid:
        out = [uid, screen_name, tid]
        out.extend(parse_header(cardwrap.find('div', class_='permalink-header')))
        out.extend(parse_text(cardwrap.find('div', class_="js-tweet-text-container")))
        out.extend(parse_fixer(cardwrap.find('div', class_='js-tweet-details-fixer tweet-details-fixer')))
        return [out]
    else:
        return [[]]


import numpy as np
import cmath


class NMF:

    def __init__(self, sparseness_W=None, sparseness_H=None, initialization=None, **control):
        self.sparseness_W = sparseness_W
        self.sparseness_H = sparseness_H

        self._stepsize_W = control.get('stepsize_W', .15)
        self._stepsize_H = control.get('stepsize_H', .15)
        self._iteration_criterion = control.get('iteration_criterion', 1e-200)
        self._eps = control.get('eps', 1e-8)
        self._Z = control.get('Z', [])

        pass

    def initialize(self):
        pass

    def decompose(self, V, r):
        n, k = V.shape
        # TODO initialize SVD
        W = np.random.standard_normal((n, r)) ** 2
        H = np.random.standard_normal((r, k)) ** 2


        if not self.sparseness_W:
            self._project(W.T, self.sparseness_W)
        if not self.sparseness_H:
            self._project(H, self.sparseness_H)

        # TODO record objhistory

        # start iteration (step 4 on page 1462)
        iter = 0
        while True:
            iter += 1

            if not self.sparseness_W:
                W -= self._stepsize_W * np.dot((np.dot(W, H) - V), H.T)
                error = 0.5 * sum(sum((V - np.dot(W, H)) ** 2))
                while True:
                    self._project(W.T, self.sparseness_W)
                    if error > 0.5 * sum(sum((V - np.dot(W, H)) ** 2)):
                        break
                    self._stepsize_W /= 2.
                    if self._stepsize_W < self._iteration_criterion:
                        return W, H
                    self._stepsize_W *= 1.2
            else:
                W *= np.dot(V, H.T) / (np.dot(W, np.dot(H, H.T)) + self._eps)

            if not self.sparseness_H:
                H -= self._stepsize_H * np.dot(W.T, np.dot(W, H) - V)
                error = 0.5 * sum(sum((V - np.dot(W, H)) ** 2))
                while True:
                    self._project(H, self.sparseness_H)
                    if error > 0.5 * sum(sum((V - np.dot(W, H)) ** 2)):
                        break
                    self._stepsize_H /= 2.
                    if self._stepsize_H < self._iteration_criterion:
                        return W, H
                    self._stepsize_H *= 1.2
            else:
                H *= np.dot(W.T, V) / (np.dot(np.dot(W.T, W), H) + self._eps)



    def _project(self, X, sparseness):
        for counter, row in enumerate(X):
            L2 = np.sqrt(np.dot(row, row))
            X[counter] = self._project(row, self._sparseness(sparseness, row, 1.), 1.)

    def _project_per_column(self, x, L1, L2):
        # Given any vector x, find the closest non-negative vector s with a given L1 and L2 norm.
        n = len(x)

        s = x + (L1 - x.sum()) / n
        # TODO: introduce initial zero matrix
        Z = np.zeros(n, bool)

        while True:
            m = (~Z) * L1 / (n - Z.sum())
            alpha = self._solve_alpha(s, m, L2)
            s = m + alpha * (s - m)
            negs = (s < 0)

            if not negs.any():
                return s

            Z |= negs
            s[Z] = 0
            c = (sum(s) - L1) / (~Z).sum()
            s -= c * (~Z)

    @staticmethod
    def _sparseness(s, x, L2):
        # TODO review page 1460
        sn = np.sqrt(len(x))
        return L2 * (s + sn - s * sn)

    @staticmethod
    def _solve_alpha(s, m, L2):
        w = s - m
        a = sum((w ** 2))
        b = 2 * np.dot(w, s.T)
        c = sum(s ** 2) - L2
        alpha = (-b + cmath.sqrt(b ** 2 - 4 * a * c).real) / (2 * a)
        return alpha





class PrintTfidfShape:

    def transform(self, X, y=None, **fit_params):
        print X.shape
        return X

    def fit_transform(self, X, y=None, **fit_params):
        self.fit(X, y, **fit_params)
        return self.transform(X)

    def fit(self, X, y=None, **fit_params):
        return self


class DenseTransformer:

    def transform(self, X, y=None, **fit_params):
        return X.todense()

    def fit_transform(self, X, y=None, **fit_params):
        self.fit(X, y, **fit_params)
        return self.transform(X)

    def fit(self, X, y=None, **fit_params):
        return self


"""
Home facts:
    Home type
    Beds, Full baths, 3/4 baths, 1/2 baths, 1/4 baths
    Finished squared feet
    Lot size
    Year Built
    Structural remodel year
    Openhouse
    Additional Info


"""

